@@ -4,6 +4,7 @@ use strict;
 use warnings;
 
 sub text  { 'filtered' }
+sub description { 'Email rejected due to a header content after SMTP DATA command' } 
 sub match {
     # Try to match that the given text and regular expressions
     # @param    [String] argv1  String to be matched with regular expressions
@@ -20,6 +21,7 @@ sub match {
             being[ ]accepted[ ]for[ ]this[ ]mailbox
         |has[ ]restricted[ ]SMS[ ]e-mail    # AT&T
         |http://postmaster[.]facebook[.]com/.+refused[ ]due[ ]to[ ]recipient[ ]preferences # Facebook
+        |IS[ ]NOT[ ]ACCEPTING[ ]ANY[ ]MAIL
         |permanent[ ]failure[ ]for[ ]one[ ]or[ ]more[ ]recipients[ ][(].+:blocked[)]
         |RESOLVER[.]RST[.]NotAuthorized # Microsoft Exchange
         |This[ ]account[ ]is[ ]protected[ ]by
@@ -103,7 +105,7 @@ Sisimai::Reason::Filtered - Bounce reason is C<filtered> or not.
 
 =head1 DESCRIPTION
 
-Sisimai::Reason::Filtered checks the bounce reason is C<filtered> or not.  This
+Sisimai::Reason::Filtered checks the bounce reason is C<filtered> or not. This
 class is called only Sisimai::Reason class.
 
 This is the error that an email has been rejected by a header content after 
@@ -145,7 +147,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2015 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2016 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
